clear
function [Cc,Kn]=Cunningham(R,P,T)
  lambda=66.5E-9*(101300/P)*(T/293.15)*(1+110/293.15)/(1+110/T);
  Kn=lambda*ones(R)./R;
  Cc=1+A*Kn+B*Kn.*exp(-C*ones(Kn)./Kn);
endfunction


function [err]=CalculNp(Np,Dm,Dp,Ki,CcDp,Df,CcDm)
  err=Dm*CcDp-Dp*CcDm*Np^(Ki/Df);
endfunction


function [Ki]=CalculKi(Dp,P,T)
    [CcDp,KnDp]=Cunningham(Dp/2,P,T)
    Ki=1.378/2*(1+erf((KnDp+4.454)/10.628));
endfunction


function [err]=CalculParam(p,mExpTab,DmExpTab,DEExpTab,Dp)
    rho=p(1);
    Df=p(2)
    NpextTab=mExpTab/(rho*%pi/6*Dp^3);
    [CcDp]=Cunningham(Dp/2,P,T);
    CcDm=Cunningham(DmExpTab/2,P,T)
    DEModelTAB=rho*(DmExpTab/Dp).^(Df/Ki-3).*(CcDm/CcDp).^(-Df/Ki);
    err=DEExpTab/rho-DEModelTAB/rho
endfunction


A=1.142; B=0.558; C=0.999;  //Définition Slip factor
T=300; //T en K
mu=1.458E-6*T^(3/2)/(110.4+T);//viscositee dynamique (ou absolue) de l'air 
k=1.38066E-23; //constante de boltzmann
rho0=1E3; //densité de référence
P=101300; //pression en Pa

[FileName,PathName,FilterIndex]=uigetfile('*','','File containing measurements')
chdir(PathName)
Dp=evstr(x_dialog('Estimated of the equivalent mass primary diameter in nm ','20'))
Dp=Dp*1E-9;


DATA=fscanfMat(FileName);
DmExpTab=DATA(:,1)*1E-9;
mExpTab=DATA(:,2)*1E-18;
DEExpTab=mExpTab./(%pi/6*DmExpTab.^3);
[Ki]=CalculKi(Dp,P,T)
[err,p, gopt] = leastsq(CalculParam,"b",[100,1],[10000,3],[1700,1.8]) 
rho=p(1);
Df=p(2);

[CcDp,KnDp]=Cunningham(Dp/2,P,T)
[Ki]=CalculKi(Dp,P,T)
DmTab=linspace(Dp,500E-9,100)';
[CcDp]=Cunningham(Dp/2,P,T);
CcDm=Cunningham(DmTab/2,P,T)
DEModelTAB=rho*(DmTab/Dp).^(Df/Ki-3).*(CcDm/CcDp).^(-Df/Ki);
clf
plot2d(DmExpTab/Dp,[DEExpTab/rho],[-3],logflag='ll') 
plot2d(DmTab/Dp,[DEModelTAB/rho],[5],logflag='ll')
xtitle('fitting procedure of the effective density''s curve','Dm/Dp','rho_eff/rho_0')
mprintf('Result of the fit :\n\tDensity of the bulk material = %0.0f kg/m^3,\n\tFractal Dimension %0.2f\n',rho,Df)

masseExport=DEModelTAB.*(%pi/6*DmTab.^3)*1E18;
Export=[DmTab*1E9 masseExport DEModelTAB];
fprintfMat('Result_of_the_fit.txt',Export)
